@@ -51,16 +51,30 @@ void SongPropertiesDialog::setSong(Song *song)
             disconnect(ui->lineEditName, SIGNAL(textChanged(QString)), this->song, SLOT(setName(QString)));
             disconnect(ui->horizontalSliderTempo, SIGNAL(valueChanged(int)), this->song, SLOT(setTempo(int)));
             disconnect(ui->horizontalSliderTicksPerLine, SIGNAL(valueChanged(int)), this->song, SLOT(setTPL(int)));
+            disconnect(this->song, SIGNAL(tempoChanged()), this, SLOT(setTempo()));
+            disconnect(this->song, SIGNAL(ticksPerLineChanged()), this, SLOT(setTicksPerLine()));
         }
 
         this->song = song;
 
         ui->lineEditName->setText(song->name());
-        ui->horizontalSliderTempo->setValue(song->tempo());
-        ui->horizontalSliderTicksPerLine->setValue(song->ticksPerLine());
+        setTempo();
+        setTicksPerLine();
 
         connect(ui->lineEditName, SIGNAL(textChanged(QString)), song, SLOT(setName(QString)));
         connect(ui->horizontalSliderTempo, SIGNAL(valueChanged(int)), song, SLOT(setTempo(int)));
         connect(ui->horizontalSliderTicksPerLine, SIGNAL(valueChanged(int)), song, SLOT(setTPL(int)));
+        connect(this->song, SIGNAL(tempoChanged()), this, SLOT(setTempo()));
+        connect(this->song, SIGNAL(ticksPerLineChanged()), this, SLOT(setTicksPerLine()));
     }
 }
+
+void SongPropertiesDialog::setTempo()
+{
+    ui->horizontalSliderTempo->setValue(song->tempo());
+}
+
+void SongPropertiesDialog::setTicksPerLine()
+{
+    ui->horizontalSliderTicksPerLine->setValue(song->ticksPerLine());
+}
