@@ -1,7 +1,7 @@
 /*
  * tracker.cpp
  *
- * Copyright 2002-2014 Vesa Halttunen
+ * Copyright 2002-2016 Vesa Halttunen
  *
  * This file is part of Tutka.
  *
@@ -27,6 +27,8 @@
 #include "block.h"
 #include "tracker.h"
 
+QHash<int, char> Tracker::keyToNote;
+
 Tracker::Tracker(QWidget *parent) :
     QWidget(parent),
     visibleLines(0),
@@ -62,14 +64,60 @@ Tracker::Tracker(QWidget *parent) :
     selectionStartLine(-1),
     selectionEndTrack(-1),
     selectionEndLine(-1),
+    oldSelectionStartTrack(-1),
     mouseSelecting(false),
-    mouseButton(Qt::NoButton)
+    mouseButton(Qt::NoButton),
+    chordStatus(0),
+    instrument_(0),
+    octave_(3),
+    inEditMode(false),
+    inChordMode(false)
 {
     font.setStyleHint(QFont::TypeWriter);
     font.setStyleStrategy(QFont::ForceIntegerMetrics);
     calculateFontSize();
     initColors();
 
+    if (keyToNote.isEmpty()) {
+        keyToNote.insert(Qt::Key_Z, 1);
+        keyToNote.insert(Qt::Key_S, 2);
+        keyToNote.insert(Qt::Key_X, 3);
+        keyToNote.insert(Qt::Key_D, 4);
+        keyToNote.insert(Qt::Key_C, 5);
+        keyToNote.insert(Qt::Key_V, 6);
+        keyToNote.insert(Qt::Key_G, 7);
+        keyToNote.insert(Qt::Key_B, 8);
+        keyToNote.insert(Qt::Key_H, 9);
+        keyToNote.insert(Qt::Key_N, 10);
+        keyToNote.insert(Qt::Key_J, 11);
+        keyToNote.insert(Qt::Key_M, 12);
+        keyToNote.insert(Qt::Key_Q, 13);
+        keyToNote.insert(Qt::Key_Comma, 13);
+        keyToNote.insert(Qt::Key_2, 14);
+        keyToNote.insert(Qt::Key_L, 14);
+        keyToNote.insert(Qt::Key_W, 15);
+        keyToNote.insert(Qt::Key_Period, 15);
+        keyToNote.insert(Qt::Key_3, 16);
+        keyToNote.insert(Qt::Key_Odiaeresis, 16);
+        keyToNote.insert(Qt::Key_E, 17);
+        keyToNote.insert(Qt::Key_Minus, 17);
+        keyToNote.insert(Qt::Key_R, 18);
+        keyToNote.insert(Qt::Key_5, 19);
+        keyToNote.insert(Qt::Key_T, 20);
+        keyToNote.insert(Qt::Key_6, 21);
+        keyToNote.insert(Qt::Key_Y, 22);
+        keyToNote.insert(Qt::Key_7, 23);
+        keyToNote.insert(Qt::Key_U, 24);
+        keyToNote.insert(Qt::Key_I, 25);
+        keyToNote.insert(Qt::Key_9, 26);
+        keyToNote.insert(Qt::Key_O, 27);
+        keyToNote.insert(Qt::Key_0, 28);
+        keyToNote.insert(Qt::Key_P, 29);
+        keyToNote.insert(Qt::Key_Aring, 30);
+        keyToNote.insert(Qt::Key_acute, 31);
+        keyToNote.insert(Qt::Key_Delete, 0);
+    }
+
     backgroundBrush = colors[ColorBackground];
     backgroundCursorBrush = colors[ColorBackgroundCursor];
     notesBrush = colors[ColorNotes];
@@ -78,6 +126,11 @@ Tracker::Tracker(QWidget *parent) :
     initDisplay(geometry().width(), geometry().height());
 }
 
+Tracker::~Tracker()
+{
+    delete pixmap;
+}
+
 void Tracker::setTracks(int tracks)
 {
     if (this->tracks != tracks) {
@@ -89,7 +142,7 @@ void Tracker::setTracks(int tracks)
         }
 
         initDisplay(geometry().width(), geometry().height());
-        queueDraw();
+        update();
 
         emit trackChanged(leftmostTrack, tracks, visibleTracks);
     }
@@ -107,7 +160,7 @@ void Tracker::setCommandPage(int commandPage)
         commandPage_ = commandPage;
         emit commandPageChanged(commandPage_);
 
-        queueDraw();
+        drawStupid();
     }
 }
 
@@ -123,22 +176,22 @@ void Tracker::setLine(int line)
         line_ = line;
         emit lineChanged(line, block_->length(), visibleLines);
 
-        queueDraw();
+        update();
     }
 }
 
 void Tracker::setSong(Song *song)
 {
     if (song_ != NULL) {
-        disconnect(song_, SIGNAL(trackMutedOrSoloed()), this, SLOT(queueDraw()));
-        disconnect(song_, SIGNAL(trackNameChanged()), this, SLOT(queueDraw()));
+        disconnect(song_, SIGNAL(trackMutedOrSoloed()), this, SLOT(update()));
+        disconnect(song_, SIGNAL(trackNameChanged()), this, SLOT(update()));
     }
 
     song_ = song;
 
     if (song_ != NULL) {
-        connect(song_, SIGNAL(trackMutedOrSoloed()), this, SLOT(queueDraw()));
-        connect(song_, SIGNAL(trackNameChanged()), this, SLOT(queueDraw()));
+        connect(song_, SIGNAL(trackMutedOrSoloed()), this, SLOT(update()));
+        connect(song_, SIGNAL(trackNameChanged()), this, SLOT(update()));
     }
 }
 
@@ -179,7 +232,7 @@ void Tracker::setBlock(unsigned int number)
                 emit commandPageChanged(commandPage_);
             }
         }
-        queueDraw();
+        drawStupid();
     }
 }
 
@@ -191,7 +244,7 @@ void Tracker::setLeftmostTrack(int leftmostTrack)
         }
 
         this->leftmostTrack = leftmostTrack;
-        queueDraw();
+        drawStupid();
 
         if (cursorTrack_ < leftmostTrack) {
             cursorTrack_ = leftmostTrack;
@@ -229,8 +282,11 @@ void Tracker::stepCursorItem(int direction)
         stepCursorTrack(direction);
         cursorItem_ = 6;
     } else {
+        if (song_ == NULL && cursorItem_ > 0 && cursorItem_ < 3) {
+            cursorItem_ = direction > 0 ? 3 : 0;
+        }
         setVisibleArea();
-        queueDraw();
+        update();
     }
 }
 
@@ -238,7 +294,7 @@ void Tracker::setCursorItem(int cursorItem)
 {
     cursorItem_ = cursorItem % 7;
     setVisibleArea();
-    queueDraw();
+    update();
 }
 
 void Tracker::stepCursorTrack(int direction)
@@ -256,23 +312,11 @@ void Tracker::stepCursorTrack(int direction)
     }
 
     setVisibleArea();
-    queueDraw();
+    update();
 
     emit cursorTrackChanged(cursorTrack_);
 }
 
-void Tracker::stepCursorLine(int direction)
-{
-    int line = line_ + direction;
-
-    while (line < 0) {
-        line += block_->length();
-    }
-    line %= block_->length();
-
-    setLine(line);
-}
-
 void Tracker::markSelection(bool enable)
 {
     if (!enable) {
@@ -283,7 +327,7 @@ void Tracker::markSelection(bool enable)
         selectionStartTrack = selectionEndTrack = cursorTrack_;
         selectionStartLine = selectionEndLine = line_;
         inSelectionMode = true;
-        queueDraw();
+        update();
     }
 
     emit selectionChanged(selectionStartTrack, selectionStartLine, selectionEndTrack, selectionEndLine);
@@ -295,17 +339,27 @@ void Tracker::clearMarkSelection()
         selectionStartTrack = selectionEndTrack = -1;
         selectionStartLine = selectionEndLine = -1;
         inSelectionMode = false;
-        queueDraw();
+        update();
 
         emit selectionChanged(selectionStartTrack, selectionStartLine, selectionEndTrack, selectionEndLine);
     }
 }
 
-bool Tracker::isInSelectionMode()
+bool Tracker::isInSelectionMode() const
 {
     return inSelectionMode;
 }
 
+bool Tracker::isInEditMode() const
+{
+    return inEditMode;
+}
+
+bool Tracker::isInChordMode() const
+{
+    return inChordMode;
+}
+
 void Tracker::noteToString(unsigned char note, unsigned char instrument, unsigned char effect, unsigned char value, char *buffer)
 {
     static const char *const noteNames[128] = {
@@ -333,8 +387,8 @@ void Tracker::noteToString(unsigned char note, unsigned char instrument, unsigne
     buffer[1] = noteNames[note][1];
     buffer[2] = noteNames[note][2];
     buffer[3] = ' ';
-    buffer[4] = hexMap[(instrument & 0xf0) >> 4];
-    buffer[5] = hexMap[instrument & 0x0f];
+    buffer[4] = song_ != NULL ? hexMap[(instrument & 0xf0) >> 4] : ' ';
+    buffer[5] = song_ != NULL ? hexMap[instrument & 0x0f] : ' ';
     buffer[6] = ' ';
     buffer[7] = hexMap[(effect & 0xf0) >> 4];
     buffer[8] = hexMap[effect & 0x0f];
@@ -530,58 +584,68 @@ void Tracker::printCursor()
 
 void Tracker::drawClever(const QRect &area)
 {
-    if (!isVisible() || block_ == NULL) {
+    if (!isVisible()) {
         return;
     }
 
-    int dist = line_ - oldLine;
-    if (dist != 0) {
-        oldLine = line_;
-        int absdist = abs(dist);
-        int redrawcnt = visibleLines;
-        int y = startY;
-
-        // Scrolling less than half the tracker area so that the cursor is still visible after scrolling?
-        if (absdist <= cursorLine) {
-            // Remove the cursor row before scrolling
-            printNotes(0, (cursorLine) * fontHeight + startY, geometry().width(), fontHeight, oldLine - dist, false);
+    QPainter painter(this);
+    if (block_ != NULL) {
+        // If selection has been added or removed, redraw everything
+        if (selectionStartTrack != oldSelectionStartTrack) {
+            oldLine = -2 * visibleLines;
+            oldSelectionStartTrack = selectionStartTrack;
         }
 
-        // Scroll the stuff already drawn on the screen
-        if (absdist < visibleLines) {
-            QPainter painter(pixmap);
-            if (dist > 0) {
-                // go down in pattern -- scroll up
-                redrawcnt = absdist;
-                painter.drawPixmap(0, y, *pixmap, 0, y + (absdist * fontHeight), geometry().width(), (visibleLines - absdist) * fontHeight);
-                y += (visibleLines - absdist) * fontHeight;
-            } else if (dist < 0) {
-                // go up in pattern -- scroll down
-                redrawcnt = absdist;
-                painter.drawPixmap(0, y + (absdist * fontHeight), *pixmap, 0, y, geometry().width(), (visibleLines - absdist) * fontHeight);
+        int dist = line_ - oldLine;
+        if (dist != 0) {
+            oldLine = line_;
+            int absdist = abs(dist);
+            int redrawcnt = visibleLines;
+            int y = startY;
+
+            // Scrolling less than half the tracker area so that the cursor is still visible after scrolling?
+            if (absdist <= cursorLine) {
+                // Remove the cursor row before scrolling
+                printNotes(0, (cursorLine) * fontHeight + startY, geometry().width(), fontHeight, oldLine - dist, false);
             }
+
+            // Scroll the stuff already drawn on the screen
+            if (absdist < visibleLines) {
+                if (dist > 0) {
+                    // go down in pattern -- scroll up
+                    redrawcnt = absdist;
+                    pixmap->scroll(0, -(absdist * fontHeight), 0, y, geometry().width(), visibleLines * fontHeight);
+                    y += (visibleLines - absdist) * fontHeight;
+                } else if (dist < 0) {
+                    // go up in pattern -- scroll down
+                    redrawcnt = absdist;
+                    pixmap->scroll(0, (absdist * fontHeight), 0, y, geometry().width(), visibleLines * fontHeight);
+                }
+            }
+
+            // Print the new rows that are now visible
+            printNotes(0, y, geometry().width(), redrawcnt * fontHeight, oldLine, true);
         }
 
-        // Print the new rows that are now visible
-        printNotes(0, y, geometry().width(), redrawcnt * fontHeight, oldLine, true);
+        // Redraw the cursor row to include the cursor
+        printNotes(0, (cursorLine) * fontHeight + startY, geometry().width(), fontHeight, line_, true);
+        printCursor();
+
+        // Print the channel headers and separator bars
+        printTrackHeaders();
+        printBars();
+
+        painter.drawPixmap(area.x(), area.y(), area.width(), area.height(), *pixmap);
+    } else {
+        painter.fillRect(area.x(), area.y(), area.width(), area.height(), isEnabled() ? backgroundBrush : QPalette().shadow());
     }
 
-    // Redraw the cursor row to include the cursor
-    printNotes(0, (cursorLine) * fontHeight + startY, geometry().width(), fontHeight, line_, true);
-    printCursor();
-
-    // Print the channel headers and separator bars
-    printTrackHeaders();
-    printBars();
-
-    QPainter painter(this);
-    painter.drawPixmap(area.x(), area.y(), area.width(), area.height(), *pixmap);
 }
 
 void Tracker::drawStupid(const QRect &area)
 {
     oldLine = -2 * visibleLines;
-    update(area);
+    update(area.isEmpty() ? QRect(QPoint(), geometry().size()) : area);
 }
 
 void Tracker::initDisplay(int width, int height)
@@ -595,6 +659,7 @@ void Tracker::initDisplay(int width, int height)
     }
     cursorLine = visibleLines / 2;
     startY = (height - fontHeight * visibleLines) / 2 + fontHeight;
+    oldLine = -2 * visibleLines;
 
     trackWidth = TRACKER_TRACK_WIDTH * fontWidth;
     int u = width - lineNumbersSpace - 10;
@@ -626,7 +691,7 @@ void Tracker::reset()
     block_ = NULL;
     initDisplay(geometry().width(), geometry().height());
     setVisibleArea();
-    queueDraw();
+    update();
 }
 
 void Tracker::initColors()
@@ -699,9 +764,9 @@ void Tracker::mouseToCursorPos(int x, int y, int *cursorTrack, int *cursorItem,
         if (*cursorItem < 4) {
             *cursorItem = 0;
         } else if (*cursorItem == 4) {
-            *cursorItem = 1;
+            *cursorItem = song_ != NULL ? 1 : 0;
         } else if (*cursorItem == 5 || *cursorItem == 6) {
-            *cursorItem = 2;
+            *cursorItem = song_ != NULL ? 2 : 3;
         } else if (*cursorItem == 7) {
             *cursorItem = 3;
         } else if (*cursorItem == 8 || *cursorItem == 9) {
@@ -752,7 +817,7 @@ void Tracker::mousePressEvent(QMouseEvent *event)
             selectionEndLine = selectionStartLine;
             selectionEndTrack = selectionStartTrack;
             mouseSelecting = true;
-            queueDraw();
+            update();
             emit selectionChanged(selectionStartTrack, selectionStartLine, selectionEndTrack, selectionEndLine);
         } else if (mouseButton == Qt::RightButton) {
             // Tracker cursor posititioning and clear block mark if any
@@ -760,7 +825,7 @@ void Tracker::mousePressEvent(QMouseEvent *event)
                 selectionStartTrack = selectionEndTrack = -1;
                 selectionStartLine = selectionEndLine = -1;
                 inSelectionMode = false;
-                queueDraw();
+                update();
                 emit selectionChanged(selectionStartTrack, selectionStartLine, selectionEndTrack, selectionEndLine);
             }
             mouseToCursorPos(x, y, &cursorTrack, &cursorItem, &line);
@@ -770,9 +835,9 @@ void Tracker::mousePressEvent(QMouseEvent *event)
                 setVisibleArea();
             }
             if (line != line_) {
-                setLine(line);
+                emit setLineRequested(line);
             }
-            queueDraw();
+            update();
         }
     }
 
@@ -801,11 +866,11 @@ void Tracker::mouseMoveEvent(QMouseEvent *event)
             setLeftmostTrack(leftmostTrack - 1);
         }
         if ((selectionEndLine > line_ + (visibleLines / 2)) || (y > geometry().height() && line_ < selectionEndLine)) {
-            setLine(line_ + 1);
+            emit setLineRequested(line_ + 1);
         } else if ((selectionEndLine < line_ - (visibleLines / 2)) || (y <= 0 && line_ > selectionEndLine)) {
-            setLine(line_ - 1);
+            emit setLineRequested(line_ - 1);
         }
-        queueDraw();
+        update();
         emit selectionChanged(selectionStartTrack, selectionStartLine, selectionEndTrack, selectionEndLine);
     }
 
@@ -823,6 +888,9 @@ void Tracker::mouseReleaseEvent(QMouseEvent *event)
 
 void Tracker::paintEvent(QPaintEvent *event)
 {
+    if (inSelectionMode || mouseSelecting) {
+        oldLine = -2 * visibleLines;
+    }
     drawClever(event->rect());
 }
 
@@ -830,7 +898,7 @@ void Tracker::showEvent(QShowEvent *event)
 {
     QWidget::showEvent(event);
 
-    drawStupid(QRect(QPoint(), geometry().size()));
+    drawStupid();
 }
 
 void Tracker::resizeEvent(QResizeEvent *event)
@@ -838,13 +906,249 @@ void Tracker::resizeEvent(QResizeEvent *event)
     QWidget::resizeEvent(event);
 
     initDisplay(geometry().width(), geometry().height());
-    queueDraw();
+    drawStupid();
 }
 
-void Tracker::queueDraw()
+void Tracker::keyPressEvent(QKeyEvent *event)
 {
-    oldLine = -2 * visibleLines;
-    update();
+    QWidget::keyPressEvent(event);
+
+    bool shift = (event->modifiers() & Qt::ShiftModifier) != 0;
+    bool ctrl = (event->modifiers() & Qt::ControlModifier) != 0;
+    bool alt = (event->modifiers() & Qt::AltModifier) != 0;
+
+    if (ctrl) {
+        switch (event->key()) {
+        case Qt::Key_B:
+            // CTRL-B: Selection mode on/off
+            markSelection(!inSelectionMode);
+            break;
+        case Qt::Key_K:
+            // CTRL-K: Clear until the end of the track
+            block_->clear(cursorTrack_, line_, cursorTrack_, block_->length() - 1);
+            break;
+        case Qt::Key_Tab:
+            // CTRL-Tab: Next command page
+            setCommandPage(commandPage_ + 1);
+            break;
+        case Qt::Key_Backtab:
+            // CTRL-Shift-Tab: Previous command page
+            setCommandPage(commandPage_ - 1);
+            break;
+        case Qt::Key_Delete: {
+            if (inEditMode) {
+                // Delete command and refresh
+                for (int i = 0; i < block_->commandPages(); i++) {
+                    block_->setCommandFull(line_, cursorTrack_, i, 0, 0);
+                }
+                emit lineEdited();
+            }
+            break;
+        }
+        }
+    } else if (shift) {
+        switch (event->key()) {
+        case Qt::Key_Backtab:
+            // Shift-Tab: Previous track
+            stepCursorTrack(-1);
+            break;
+        case Qt::Key_Delete:
+            // Delete note+command and refresh
+            if (inEditMode) {
+                block_->setNote(line_, cursorTrack_, 0, 0, 0);
+                for (int commandPage = 0; commandPage < block_->commandPages(); commandPage++) {
+                    block_->setCommandFull(line_, cursorTrack_, commandPage, 0, 0);
+                }
+                emit lineEdited();
+            }
+            break;
+        case Qt::Key_Backspace:
+            // Shift-Backspace: Insert row
+            if (inEditMode) {
+                block_->insertLine(line_, alt ? -1 : cursorTrack_);
+            }
+            break;
+        default:
+            break;
+        }
+    } else if (alt) {
+        switch (event->key()) {
+        case Qt::Key_Backspace:
+            // Backspace: delete line
+            if (inEditMode) {
+                block_->deleteLine(line_);
+            }
+            break;
+        default:
+            break;
+        }
+    } else {
+        char note = -1;
+        char data = -1;
+
+        if (cursorItem_ == 0) {
+            // Editing notes
+            data = keyToNote.value(event->key(), -1);
+
+            if (data >= 0) {
+                note = data - 1;
+
+                if (inChordMode) {
+                    // Chord mode: check if the key has already been pressed (key repeat)
+                    bool found = keyboardKeysDown.contains(event->key());
+
+                    // Go to next channel if the current key has not been pressed down yet
+                    if (!found) {
+                        // Add the key to the keys down list
+                        keyboardKeysDown.append(event->key());
+
+                        if (inEditMode) {
+                            // Set note and refresh
+                            block_->setNote(line_, cursorTrack_, octave_, data, instrument_);
+                        }
+
+                        addChordNote();
+                    }
+                } else if (inEditMode) {
+                    // Set note and refresh
+                    block_->setNote(line_, cursorTrack_, octave_, data, instrument_);
+                    emit lineEdited();
+                }
+            }
+        } else if (inEditMode) {
+            switch (cursorItem_) {
+            case 1:
+            case 2:
+                // Editing instrument
+                if (event->key() >= Qt::Key_0 && event->key() <= Qt::Key_9) {
+                    data = event->key() - Qt::Key_0;
+                } else if (event->key() >= Qt::Key_A && event->key() <= Qt::Key_F) {
+                    data = 10 + event->key() - Qt::Key_A;
+                } else if (event->key() == Qt::Key_Delete) {
+                    data = 0;
+                }
+
+                if (data >= 0) {
+                    // Set instrument and refresh
+                    int ins = block_->instrument(line_, cursorTrack_);
+                    if (cursorItem_ == 1) {
+                        ins = (ins & 0x0f) | (data << 4);
+                    } else {
+                        ins = (ins & 0xf0) | data;
+                    }
+                    block_->setInstrument(line_, cursorTrack_, ins);
+                    emit lineEdited();
+                }
+                break;
+            case 3:
+            case 4:
+            case 5:
+            case 6:
+                // Editing effects
+                if (event->key() >= Qt::Key_0 && event->key() <= Qt::Key_9) {
+                    data = event->key() - Qt::Key_0;
+                } else if (event->key() >= Qt::Key_A && event->key() <= Qt::Key_F) {
+                    data = 10 + event->key() - Qt::Key_A;
+                } else if (event->key() == Qt::Key_Delete) {
+                    data = 0;
+                }
+
+                if (data >= 0) {
+                    // Set effect and refresh
+                    block_->setCommand(line_, cursorTrack_, commandPage_, cursorItem_ - 3, data);
+                    emit lineEdited();
+                }
+                break;
+            }
+        }
+
+        switch (event->key()) {
+        case Qt::Key_F1:
+        case Qt::Key_F2:
+        case Qt::Key_F3:
+        case Qt::Key_F4:
+        case Qt::Key_F5:
+        case Qt::Key_F6:
+        case Qt::Key_F7:
+        case Qt::Key_F8:
+        case Qt::Key_F9:
+        case Qt::Key_F10:
+        case Qt::Key_F11:
+            // F1-F11: Set active keyboard octave
+            setOctave(event->key() - Qt::Key_F1);
+            break;
+        case Qt::Key_Left:
+            // Left: Go left
+            stepCursorItem(-1);
+            break;
+        case Qt::Key_Right:
+            // Right: Go right
+            stepCursorItem(1);
+            break;
+        case Qt::Key_Down:
+            // Down: Go down
+            emit setLineRequested(line_ + 1);
+            break;
+        case Qt::Key_Up:
+            // Up: Go up
+            emit setLineRequested(line_ - 1);
+            break;
+        case Qt::Key_Home:
+            // Home: Go to the beginning of block
+            emit setLineRequested(0);
+            break;
+        case Qt::Key_End:
+            // End: Go to the end of block
+            emit setLineRequested(block_->length() - 1);
+            break;
+        case Qt::Key_PageDown:
+            // Page down: Go down 8 lines
+            emit setLineRequested(line_ + 8);
+            break;
+        case Qt::Key_PageUp:
+            // Page up: Go up 8 lines
+            emit setLineRequested(line_ - 8);
+            break;
+        case Qt::Key_Tab:
+            // Tab: Next track
+            setCursorItem(0);
+            stepCursorTrack(1);
+            break;
+        case Qt::Key_Backspace:
+            if (inEditMode) {
+                // Backspace: delete line
+                block_->deleteLine(line_, cursorTrack_);
+            }
+            break;
+        default:
+            break;
+        }
+
+        // Play note if a key was pressed but not if cursor is in cmd pos
+        if (note >= 0 && cursorItem_ == 0) {
+            emit notePressed(note);
+        }
+    }
+}
+
+void Tracker::keyReleaseEvent(QKeyEvent *event)
+{
+    QWidget::keyReleaseEvent(event);
+
+    bool shift = (event->modifiers() & Qt::ShiftModifier) != 0;
+    bool ctrl = (event->modifiers() & Qt::ControlModifier) != 0;
+
+    // Key has been released
+    if (!ctrl && !shift) {
+        if (inChordMode && cursorItem_ == 0) {
+            if (event->key() != Qt::Key_Delete && keyToNote.contains(event->key())) {
+                // Find the key from the keys down list and remove it
+                keyboardKeysDown.removeAll(event->key());
+
+                removeChordNote();
+            }
+        }
+    }
 }
 
 Song *Tracker::song() const
@@ -882,6 +1186,11 @@ int Tracker::cursorItem() const
     return cursorItem_;
 }
 
+int Tracker::octave() const
+{
+    return octave_;
+}
+
 void Tracker::redrawArea(int startTrack, int startLine, int endTrack, int endLine)
 {
     Q_UNUSED(startTrack)
@@ -889,14 +1198,15 @@ void Tracker::redrawArea(int startTrack, int startLine, int endTrack, int endLin
     Q_UNUSED(endTrack)
     Q_UNUSED(endLine)
 
-    queueDraw();
+    // TODO: redraw only the changed area
+    drawStupid();
 }
 
 void Tracker::checkBounds()
 {
     if (cursorTrack_ >= block_->tracks()) {
         cursorTrack_ = block_->tracks() - 1;
-        queueDraw();
+        update();
     }
 
     if (line_ >= block_->length()) {
@@ -915,6 +1225,48 @@ void Tracker::setSelection(int startTrack, int startLine, int endTrack, int endL
     selectionEndTrack = endTrack;
     selectionEndLine = endLine;
     inSelectionMode = false;
-    queueDraw();
+    update();
     emit selectionChanged(selectionStartTrack, selectionStartLine, selectionEndTrack, selectionEndLine);
 }
+
+void Tracker::setEditMode(bool enabled)
+{
+    inEditMode = enabled;
+}
+
+void Tracker::setChordMode(bool enabled)
+{
+    inChordMode = enabled;
+}
+
+void Tracker::setInstrument(int instrument)
+{
+    instrument_ = instrument;
+}
+
+void Tracker::setOctave(int octave)
+{
+    if (octave_ != octave) {
+        octave_ = octave;
+
+        emit octaveChanged(octave_);
+    }
+}
+
+void Tracker::addChordNote()
+{
+    stepCursorTrack(1);
+    chordStatus++;
+}
+
+void Tracker::removeChordNote()
+{
+    // Go to the previous channel
+    stepCursorTrack(-1);
+    chordStatus--;
+
+    // If all chord notes have been released go to the next line
+    if (chordStatus == 0 && inEditMode) {
+        emit lineEdited();
+    }
+}
